@@ -1,5 +1,7 @@
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals, absolute_import, division, print_function
 
-__version__ = '0.21.0'
+__version__ = '0.22.1'
 
 from poker._common import PokerEnum
 from poker.card import Suit, Rank, Card, FACE_RANKS, BROADWAY_RANKS
